/*
 * ArcVis - demo for the UT-Vienna Visualisierung 2 VU course
 * (http://www.cg.tuwien.ac.at/courses/Visualisierung2/)
 * Copyright (C) 2015 Adam Celarek (www.celarek.at)
 * Copyright (C) 2015 Daniel Steinboeck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Arc.h"

#include <algorithm>
#include <QPainter>


Arc::Arc(float start, float width, float end, QString subString) : m_start(start), m_width(width), m_end(end), m_substring(subString)
{
    m_end = std::max(m_end, m_start);
    m_width = std::min(m_width, (m_end - m_start) * 0.5f);
}

QRectF Arc::boundingRect() const
{
    QPointF topLeft(m_start, 0);
    QSizeF boundingSize(m_end-m_start, (m_end-m_start) * 0.5f);
    return QRectF(topLeft, boundingSize);
}

QPainterPath Arc::painterPath() const
{
    QPainterPath path;
    path.moveTo(m_start, 0);
    float bigArcWidth = m_end - m_start;
    QRectF bigArc(m_start, -bigArcWidth * 0.5f, bigArcWidth, bigArcWidth);
    path.arcTo(bigArc, 180, 180);
    path.lineTo(m_end-m_width, 0);
    float smallArcWidth = m_end - m_start - 2.f * m_width;
    QRectF smallArc(m_start+m_width, -smallArcWidth * 0.5f, smallArcWidth, smallArcWidth);
    path.arcTo(smallArc, 0, -180);
    path.closeSubpath();

    return path;
}

void Arc::paint(QPainter *painter, const QColor fillColour)
{
    painter->fillPath(painterPath(), fillColour);
//    painter->setPen(QPen(QColor(0, 0, 0), 0.1f ));
//    painter->drawPath(path);
}
