/*
 * ArcVis - demo for the UT-Vienna Visualisierung 2 VU course
 * (http://www.cg.tuwien.ac.at/courses/Visualisierung2/)
 * Copyright (C) 2015 Adam Celarek (www.celarek.at)
 * Copyright (C) 2015 Daniel Steinboeck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ARCITEM_H
#define ARCITEM_H

#include <QPainterPath>

class Arc
{
public:
    /**
     * @brief Arc represents one arc.
     *
     * It holds the corresponding data and can draw itself.
     * @param start Where the arc starts, eg most left point
     * @param width Width at the start, eg the arc starts from [start, start + width]
     * @param end The last point of the arc end, eg it ends at [end - width, end]
     * @param subString The text, which was used to generate this arc. Used to find arcs belonging to the same substring.
     */
    Arc(float start, float width, float end, QString subString);

    /**
     * @brief boundingRect is a utility function for ArcGroupItem
     * @return The bounding rect as defined in Qt's QGraphicsItem.
     */
    QRectF boundingRect() const;
    /**
     * @brief painterPath returns a QPainterPath representing the arc.
     *
     * It is then used for painting and for highlighting by mouse.
     *
     * @return the QPainterPath representing the arc
     */
    QPainterPath painterPath() const;

    /**
     * @brief paint this arc
     * @param painter use this QPainter for painting
     * @param fillColour draw with the given colour
     */
    void paint(QPainter *painter, const QColor fillColour);

    /**
     * @brief substring returns the substring, that was used to generate this arc.
     * @return the substring
     */
    QString substring() const { return m_substring; }

private:
    float m_start, m_width, m_end;
    QString m_substring;
};

#endif // ARCITEM_H
