/*
 * ArcVis - demo for the UT-Vienna Visualisierung 2 VU course
 * (http://www.cg.tuwien.ac.at/courses/Visualisierung2/)
 * Copyright (C) 2015 Adam Celarek (www.celarek.at)
 * Copyright (C) 2015 Daniel Steinboeck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "ArcGroupItem.h"

#include <QPainter>
#include <QStyleOptionGraphicsItem>
#include <QStatusBar>

ArcGroupItem::ArcGroupItem(QString substring, QColor color, MainWindow *mainWindow) : m_substring(substring), m_colour(color), m_mainWindow(mainWindow)
{
//    setFlags(ItemIsSelectable|QGraphicsItem::ItemIsFocusable);
    setAcceptHoverEvents(true);
    setZValue(-m_substring.length());
}

QRectF ArcGroupItem::boundingRect() const
{
    return m_boundingRect;
}

void ArcGroupItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{

    Q_UNUSED(widget);

    QColor fillColour = (option->state & QStyle::State_Selected) ? m_colour.darker(200) : m_colour;
    if (option->state & QStyle::State_MouseOver)
        fillColour = QColor(255, 120, 0, 100);
//        fillColour = fillColour.lighter(125);

//    fillColour.setAlpha(100);
//    painter->fillPath(m_painterPath, fillColour);
    foreach (Arc arc, m_items) {
        arc.paint(painter, fillColour);
    }
}

QPainterPath ArcGroupItem::shape() const
{
    return m_painterPath;
}

void ArcGroupItem::hoverMoveEvent(QGraphicsSceneHoverEvent *event)
{
    m_mainWindow->statusBar()->showMessage(QString("Substring: ") + m_substring, 1000);
    QGraphicsItem::hoverMoveEvent(event);
}

void ArcGroupItem::addArc(Arc item)
{
    m_items.append(item);
    m_boundingRect = m_boundingRect.united(item.boundingRect());
    m_painterPath = m_painterPath.united(item.painterPath());
}

void ArcGroupItem::setColour(QColor colour)
{
    m_colour = colour;
}

