/*
 * ArcVis - demo for the UT-Vienna Visualisierung 2 VU course
 * (http://www.cg.tuwien.ac.at/courses/Visualisierung2/)
 * Copyright (C) 2015 Adam Celarek (www.celarek.at)
 * Copyright (C) 2015 Daniel Steinboeck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "MyGraphicsView.h"

#include <QWheelEvent>
#include <QtMath>

MyGraphicsView::MyGraphicsView(QWidget *parent) : QGraphicsView(parent)
{

}

void MyGraphicsView::mousePressEvent(QMouseEvent *e)
{
    if(e->button() == Qt::MiddleButton) {
        setDragMode(QGraphicsView::ScrollHandDrag);
        QMouseEvent* newEvent = new QMouseEvent(e->type(), e->localPos(), e->windowPos(), e->screenPos(), Qt::LeftButton, e->buttons() | Qt::LeftButton, e->modifiers());
        QGraphicsView::mousePressEvent(newEvent);
        e->setAccepted(newEvent->isAccepted());
        delete newEvent;
    }
    QGraphicsView::mousePressEvent(e);
}

void MyGraphicsView::mouseReleaseEvent(QMouseEvent *e)
{
    if(e->button() == Qt::MiddleButton) {
        setDragMode(QGraphicsView::NoDrag);
        QMouseEvent* newEvent = new QMouseEvent(e->type(), e->localPos(), e->windowPos(), e->screenPos(), Qt::LeftButton, e->buttons() | Qt::LeftButton, e->modifiers());
        QGraphicsView::mouseReleaseEvent(newEvent);
        e->setAccepted(newEvent->isAccepted());
        delete newEvent;
    }
    QGraphicsView::mouseReleaseEvent(e);
}

void MyGraphicsView::mouseMoveEvent(QMouseEvent *e)
{
    if((e->buttons() & Qt::MiddleButton) == Qt::MiddleButton) {
        QMouseEvent* newEvent = new QMouseEvent(e->type(), e->localPos(), e->windowPos(), e->screenPos(), Qt::NoButton, e->buttons() | Qt::LeftButton, e->modifiers());
        QGraphicsView::mouseMoveEvent(newEvent);
        e->setAccepted(newEvent->isAccepted());
        delete newEvent;
    }
    QGraphicsView::mouseMoveEvent(e);

}

void MyGraphicsView::wheelEvent(QWheelEvent* e)
{
    float scale = 1.f/1.05f;
    if (e->delta() > 0)
        scale = 1.05;

    QTransform m(scale, 0, 0, scale, 0, 0);
    setTransform(m, true);

    e->accept();
}

