/*
 * ArcVis - demo for the UT-Vienna Visualisierung 2 VU course
 * (http://www.cg.tuwien.ac.at/courses/Visualisierung2/)
 * Copyright (C) 2015 Adam Celarek (www.celarek.at)
 * Copyright (C) 2015 Daniel Steinboeck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MYGRAPHICSVIEW_H
#define MYGRAPHICSVIEW_H

#include <QGraphicsView>

/**
 * @brief The MyGraphicsView class changes some default behavior of QGraphicsView
 *
 * To be precise it adds panning with the middle mouse button and zooming with the mouse wheel.
 */
class MyGraphicsView : public QGraphicsView
{
public:
    /**
     * @brief MyGraphicsView constructs the object by calling the constructor of the super class
     * @param parent of the QWidget
     */
    MyGraphicsView(QWidget * parent = 0);
protected:
    /**
     * @brief mousePressEvent is used for changing the panning behaviour
     * @param e parameters of the QMouseEvent
     */
    void mousePressEvent(QMouseEvent* e);
    /**
     * @brief mouseReleaseEvent is used for changing the panning behaviour
     * @param e parameters of the QMouseEvent
     */
    void mouseReleaseEvent(QMouseEvent* e);
    /**
     * @brief mouseMoveEvent is used for changing the panning behaviour
     * @param e parameters of the QMouseEvent
     */
    void mouseMoveEvent(QMouseEvent* e);
    /**
     * @brief wheelEvent is used for zooming with the mouse wheel
     * @param event parameters of the QWheelEvent
     */
    void wheelEvent(QWheelEvent* event) override;

private:
    float m_scale = 0.f;
};

#endif // MYGRAPHICSVIEW_H
