/*
 * ArcVis - demo for the UT-Vienna Visualisierung 2 VU course
 * (http://www.cg.tuwien.ac.at/courses/Visualisierung2/)
 * Copyright (C) 2015 Adam Celarek (www.celarek.at)
 * Copyright (C) 2015 Daniel Steinboeck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "RepetitionRegionInterval.h"

/**
 * @brief RepetitionRegionInterval::RepetitionRegionInterval Constructor
 * @param start the start of the RepetitionRegionInterval
 * @param end the end of the RepetitionRegionInterval
 */
RepetitionRegionInterval::RepetitionRegionInterval(int start, int end){
    this->start = start;
    this->end = end;
}

/**
 * @brief RepetitionRegionInterval::RepetitionRegionInterval
 * @param text the text covered by the RepetitionRegionInterval
 * @param start the start of the RepetitionRegionInterval
 * @param end the end of the RepetitionRegionInterval
 * @param substringLength the length of the substring
 */
RepetitionRegionInterval::RepetitionRegionInterval(QStringList text, int start, int end, int substringLength){
    this->text = text;
    this->start = start;
    this->end = end;
    this->substringLength = substringLength;
}

/**
 * @brief RepetitionRegionInterval::getStart returns the start of the RepetitionRegionInterval
 * @return start of the RepetitionRegionInterval
 */
int RepetitionRegionInterval::getStart(){
    return start;
}

/**
 * @brief RepetitionRegionInterval::getEnd returns the end of the RepetitionRegionInterval
 * @return end of the RepetitionRegionInterval
 */
int RepetitionRegionInterval::getEnd(){
    return end;
}

/**
 * @brief RepetitionRegionInterval::getIntervalLength returns the length of the RepetitionRegionInterval
 * @return the length of the RepetitionRegionInterval
 */
int RepetitionRegionInterval::getIntervalLength(){
    return (end-start+1);
}

/**
 * @brief RepetitionRegionInterval::getSubstringLength returns the substring length of the RepetitionRegionInterval
 * @return the substring length of the RepetitionRegionInterval
 */
int RepetitionRegionInterval::getSubstringLength(){
    return substringLength;
}

/**
 * @brief RepetitionRegionInterval::setInterval sets the start and end of a RepetitionRegionInterval
 * @param start the start of the RepetitionRegionInterval
 * @param end the end of the RepetitionRegionInterval
 */
void RepetitionRegionInterval::setInterval(int start, int end){
    this->start = start;
    this->end = end;
}

/**
 * @brief RepetitionRegionInterval::isFundamentalSubstring checks if given RepetitionRegionInterval is a fundamental substring of this RepetitionRegionInterval
 * @param interval the RepetitionRegionInterval to check
 * @return true if fundamental substring, false if not
 */
bool RepetitionRegionInterval::isFundamentalSubstring(RepetitionRegionInterval interval){
    if(containsInterval(interval) && interval.getIntervalLength() == substringLength && (interval.getStart()-getStart())%substringLength == 0){
        return true;
    }
    else{
        return false;
    }
}

/**
 * @brief RepetitionRegionInterval::haveSameFundamentalSubstring checks if both given RepetitionRegionInterval have the same fundamental substring
 * @param i1 the first RepetitionRegionInterval to check
 * @param i2 the second RepetitionRegionInterval to check
 * @return true if they have the same fundamental substring, false if they have not
 */
bool RepetitionRegionInterval::haveSameFundamentalSubstring(RepetitionRegionInterval i1, RepetitionRegionInterval i2){
    if(i1.getIntervalLength() <= substringLength && i1.getStart() <= i2.getStart() && (i1.getStart()+substringLength-1) >= i2.getEnd()){
        return true;
    }
    else{
        return false;
    }
}

/**
 * @brief RepetitionRegionInterval::containsInterval checks if current RepetitionRegionInterval contains the given one
 * @param interval the RepetitionRegionInterval to check
 * @return true if it contains the given RepetitionRegionInterval, false if not
 */
bool RepetitionRegionInterval::containsInterval(RepetitionRegionInterval interval){
    if(interval.getStart() >= start && interval.getEnd() <= end){
        return true;
    }
    else{
        return false;
    }
}

/**
 * @brief RepetitionRegionInterval::intersectsInterval checks if current RepetitionRegionInterval intersects the given one
 * @param interval the RepetitionRegionInterval to check
 * @return true if it intersects the given RepetitionRegionInterval, false if not
 */
bool RepetitionRegionInterval::intersectsInterval(RepetitionRegionInterval interval){
    if(start > interval.getEnd() || end < interval.getStart()){
        return true;
    }
    else{
        return false;
    }
}
