/*
 * ArcVis - demo for the UT-Vienna Visualisierung 2 VU course
 * (http://www.cg.tuwien.ac.at/courses/Visualisierung2/)
 * Copyright (C) 2015 Adam Celarek (www.celarek.at)
 * Copyright (C) 2015 Daniel Steinboeck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "RepetitionRegionList.h"

/**
 * @brief RepetitionRegionList::RepetitionRegionList Contructor
 * @param text the text where repetition regions should be found
 * @param rootNode the root node of the tree
 */
RepetitionRegionList::RepetitionRegionList(QStringList text, Node rootNode){
    this->text = text;
    regions = new QVector<QList<RepetitionRegionInterval> >(text.length());
    locateRepRegIntervals(rootNode);
}

/**
 * @brief RepetitionRegionList::getRepetitionRegionIntervals get list with all RepetitionRegionIntervals at a certain position
 * @param pos the position where the repetition region starts
 * @return a list containing all RepetitionRegionIntervals at pos
 */
QList<RepetitionRegionInterval> RepetitionRegionList::getRepetitionRegionIntervals(int pos){
    return regions->at(pos);
}

/**
 * @brief RepetitionRegionList::locateRepRegIntervals locate all RepetitionRegionIntervals
 * @param rootNode the root node of the tree
 */
void RepetitionRegionList::locateRepRegIntervals(Node rootNode){
    NodeList nodeList(rootNode);
    while(nodeList.hasNextNode()){
        Node node = nodeList.nextNode();
        int n = node.getStartPosList().length();

        //repetition must have at least 2 starting positions
        if(n > 1){
            int level = node.getLevel();
            for(int i=0;i<(n-1);i++){
                if(node.getStartPos(i+1)-node.getStartPos(i) == level){
                    addRepRegInterval(node.getStartPos(i), level);
                }
            }
        }
    }
}

/**
 * @brief RepetitionRegionList::addRepRegInterval adds RepetitionRegionInterval to the RepetitionRegionList if it wasn't located yet
 * @param start the start of the repetition region
 * @param substringLength the length of the substring
 */
void RepetitionRegionList::addRepRegInterval(int start, int substringLength){
    RepetitionRegionInterval interval(start, start+substringLength);
    if(regions->size() > start){
        QList<RepetitionRegionInterval> region = regions->at(start);
        for(int i=0;i<region.length();i++){
            RepetitionRegionInterval repRegInterval = region.at(i);
            //check if already exists
            if(repRegInterval.containsInterval(interval) && substringLength == repRegInterval.getSubstringLength()){
                return;
            }
        }
    }

    //check left side
    while(start > 0 && text.at(start-1).compare(text.at(start+substringLength-1)) == 0){
        start--;
    }

    //check right side
    int end = start+substringLength;
    while(end < text.length()-1 && text.at(end+1).compare(text.at(end-substringLength+1)) == 0){
        end++;
    }

    RepetitionRegionInterval region(text, start, end, substringLength);
    for(int i=start;i<=end;i++){
        QList<RepetitionRegionInterval> r = regions->at(i);
        r.append(region);
        regions->replace(i, r);
    }
}
