/*
 * ArcVis - demo for the UT-Vienna Visualisierung 2 VU course
 * (http://www.cg.tuwien.ac.at/courses/Visualisierung2/)
 * Copyright (C) 2015 Adam Celarek (www.celarek.at)
 * Copyright (C) 2015 Daniel Steinboeck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "MainWindow.h"
#include <QApplication>


/** @mainpage Arc Visualisation of repetitions in text
 *
 * @image html screenshot1.png
 *
 * @section Overview
 * The paper %Arc Diagrams: Visualizing Structure in Strings by Martin Wattenberg introduces a new method to visualize
 * repetitive patterns in string data. The basic idea is, that complex patterns of repetition, can simply be visualized
 * by drawing an arc across them. This technique can be used to not only highlight repeated subsequences in in text,
 * but also in melodies or DNA.
 *
 * @section Implementation
 * The application is written in C++ using the Qt framework in version 5.4 for the graphical user interface. As a build
 * system we use CMake. Therefore it is possible to compile on all major computer platforms (Windows, Linux, MacOS).
 *
 * In order to find repeated substrings, we used a suffix-tree-like structure. We then used the tree to find repetition regions and
 * checked if they are essential according to the three definitions stated in the paper.
 *
 * For drawing QGraphicsScene in conjunction with QGraphicsView is used. Together with Qt's graphical user interface (GUI)
 * classes this allows for a slick desgin. Qt's GUI builder was used to create all the layout of the MainWindow. This allowed
 * a very quick development.
 *
 * @section Usage
 * Write or paste the string where you want to find repetitive patterns in, inside the text area to the left and click#
 * the Process Button on the top. On the right side you will see the output with arcs highlighting the repetitive regions.
 * If you move your cursor over an arc it will be highlighted as well as every other arc sharing the same substring. The
 * repeated substring will also be displayed in the status bar at the bottom. If you wish to visualize only repeated
 * substring of a specific length just change the option Min length to your desired length.
 *
 * If you like to see the changes in the output immediately while you altering your string or the minimum length you have
 * to tick the checkbox live update of min length and text.
 *
 * Before you export your %Arc Diagram you have the possibility to change the properties of it. You can change the color
 * and the lucency of the arcs, the background color, the baseline color and its width as well as the text color. You can
 * even disable the text if you just want to visualize the arcs by unticking the Display Text checkbox.
 *
 * If you like how your %Arc Diagram looks, you can now export it. Set the desired export width and click the Export Button
 * on the top. Enter a name and save it.
 *
 * @section Resources
 *  - [Compiled Binaries for Windows](../zips/binaries_windows.zip)
 *  - [Source Code](../zips/code.zip)
 *
 * @section References
 *  - [Arc Diagrams: Visualizing Structure in Strings](http://ieeexplore.ieee.org/xpls/abs_all.jsp?arnumber=1173155)
 *  - [Source code repository](https://bitbucket.org/adamce/tuwien_vis2/src)
 *
 * @image html arcvis.png
 */

/**
 * @brief main is the entry point of the program
 *
 * It creates the QApplication and the MainWindow, and runs the exec method of QApplication.
 * @param argc argument count given in the command line
 * @param argv vector of c strings, the arguments from the command line
 * @return an exit code provided by QApplication
 */
int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    MainWindow w;
    w.show();

    return a.exec();
}
